//подключение библиотек
#include <EEPROM.h>
#include "ESPAsyncWebServer.h"
#include <DNSServer.h>
#ifdef ESP32
#include <AsyncTCP.h>
#include <WiFi.h>
#include <HTTPClient.h>
#endif
#ifdef ESP8266
#include <ESPAsyncTCP.h>
#include <ESP8266HTTPClient.h>
#include <ESP8266WiFi.h>
#endif

//создание веб сервера и dns сервера
AsyncWebServer server(80);
DNSServer dnsServer;

//HTML разметка для режима "настройка"(английский)
const char index_html[] PROGMEM = R"rawliteral(
    <!DOCTYPE HTML><html><head>
        <meta charset="utf-8">
        <meta name="viewport" content="width=device-width, initial-scale=1">
        <title>Module 1</title>
        <style>
        body{background: #0a0c25ff;}
        .main{
            position: fixed;
            top: 0;
            left: 0;
            width: 100%;
            height: 100%;
            background: #0a0c25ff;
            display: flex;
            flex-direction: column;
            justify-content: center;
        }
        .main__inner{
            border-radius: 20px;
            background: #020311;
            margin: auto;
            padding: 20px;
            text-align: center;
            font-size: 20px;
            color: white;
            font-family: Arial, sans-serif;
        }
        </style>
    </head>
    <body>
        <div class="main">
            <div class="main__inner">
                <b style="font-size: 30px;">Setup before use<br>(Module 1)</b><br>    
                <form action="/set" style="margin-top: 20px;">
                    Ssid (name of wifi):<br><input placeholder="Ssid" type="text" name="ssid"><br><br>
                    Password (password of wifi):<br><input placeholder="Password" type="text" name="password"><br><br>
                    <input type="submit" placeholder="Submit" value="Submit">
                </form>
                <form action="/ru" style="margin-top: 10px;">
                    <input type="submit" placeholder="Submit" value="Перевести">
                </form>
            </div>
        </div>
    </body></html>)rawliteral";
//HTML разметка для режима "настройка"(русская)
const char indexRU_html[] PROGMEM = R"rawliteral(
    <!DOCTYPE HTML><html><head>
        <meta charset="utf-8">
        <meta name="viewport" content="width=device-width, initial-scale=1">
        <title>Модуль 1</title>
        <style>
            body{background: #0a0c25ff;}
            .main{
                position: fixed;
                top: 0;
                left: 0;
                width: 100%;
                height: 100%;
                background: #0a0c25ff;
                display: flex;
                flex-direction: column;
                justify-content: center;
            }
            .main__inner{
                border-radius: 20px;
                background: #020311;
                margin: auto;
                padding: 20px;
                text-align: center;
                font-size: 20px;
                color: white;
                font-family: Arial, sans-serif;
            }
        </style>
    </head>
    <body>
        <div class="main">
            <div class="main__inner">
                <b style="font-size: 30px;">Настройка перед<br>использованием<br>(Модуль 1)</b><br>
                <form action="/set" style="margin-top: 20px;">
                    Ssid (имя wifi сети):<br><input placeholder="Ssid" type="text" name="ssid"><br><br>
                    Password (пороль wifi сети):<br><input placeholder="Password" type="text" name="password"><br><br>
                    <input type="submit" placeholder="Submit" value="Submit">
                </form>
                <form action="/" style="margin-top: 10px;">
                    <input type="submit" placeholder="Submit" value="Translate">
                </form>
            </div>
        </div>
    </body></html>)rawliteral";
//настройки для dns сервера
class CaptiveRequestHandler : public AsyncWebHandler {
    public:
        CaptiveRequestHandler() {}
        virtual ~CaptiveRequestHandler() {}

        bool canHandle(AsyncWebServerRequest *request){
            //request->addInterestingHeader("ANY");
            return true;
        }

        void handleRequest(AsyncWebServerRequest *request) {
            request->send_P(200, "text/html", index_html); 
        }
};

class MainModule {
    public:
        //функция для записи данных(слова) в память
        void setE(int a, int b, String str) {
            EEPROM.write(a, str.length());
            for (int i = 0; i < str.length(); i++) {
                EEPROM.write(b + i, str[i]);
            }
            EEPROM.commit();
        }
        //функция для получение данных(слова) из память
        String getE(int a, int b) {
            String info;
            for (int i = 0; i < EEPROM.read(a); i++) {
                info += char(EEPROM.read(b + i));
            }
            return info;
        }
        //функция настройки сервера в режиме "настройка"
        void setupServer() {
            server.on("/", HTTP_GET, [](AsyncWebServerRequest * request) {
                request->send_P(200, "text/html", index_html);
            });
            server.on("/ru", HTTP_GET, [](AsyncWebServerRequest * request) {
                request->send_P(200, "text/html", indexRU_html);
            });
            server.on("/set", HTTP_GET, [](AsyncWebServerRequest * request) {
                request->send_P(200, "text/plain", "Sent");
                if (request->hasParam("ssid") && request->hasParam("password")) {
                    Serial.println(request->getParam("ssid")->value());
                    Serial.println(request->getParam("password")->value());
                    MainModule mod;
                    mod.setE(1, 2, request->getParam("ssid")->value());
                    mod.setE(19, 20, request->getParam("password")->value());
                    EEPROM.write(0, 1);
                    EEPROM.commit();
                    ESP.restart();
                }
            });
            dnsServer.start(53, "*", WiFi.softAPIP());
            server.addHandler(new CaptiveRequestHandler()).setFilter(ON_AP_FILTER);
            server.begin();
        }
};
