//подключение нужных библиотек
#include <AsyncTCP.h>
#include <Tone32.h>
#include "ESPAsyncWebServer.h"
#include <ArduinoJson.h>
#include <WiFi.h>
#include <DNSServer.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <OtherModule.h>
#include <TroykaMQ.h>

//пины куда подключены датчики
#define TEMP_PIN 4
#define FLOOD_PIN 35
#define FIRE_PIN 27
#define GAS_PIN 34
#define BUZZER_PIN 5

//инициализируем класы
OtherModule mod;

MQ2 mq2(GAS_PIN);

OneWire oneWire(TEMP_PIN);
DallasTemperature sensor(&oneWire);


//объявление перевенных
int8_t eeprom_status, reset_timer = 0;

int8_t temp, gas, flood, fire;

bool send_status = false;
bool gas_status = false;

int intervalSend = 0;
int intervalCheck = 0;
DynamicJsonDocument answer(512);



//=====

void setup() {
  //инициализируем работу Serial-порта
  Serial.begin(9600);
  //инициализируем работу EEPROM
  EEPROM.begin(128);
  //берём режим из памяти платы
  eeprom_status = EEPROM.read(0);
  //выводим режим в Serial
  Serial.print("eeprom_status: ");
  Serial.println(eeprom_status);
  //если режим 1 - "работа"; если нет - "настройка"
  if (eeprom_status == 1) {
    //режим работы wifi
    WiFi.mode(WIFI_STA);
    // подключаемся к wifi
    WiFi.begin(mod.getE(1, 2).c_str(), mod.getE(19, 20).c_str());
    //цикл пока не подключимся к wifi
    while (WiFi.status() != WL_CONNECTED) {
      //звуковой индекатор
      tone(BUZZER_PIN, NOTE_B2, 100, 0);
      delay(200);
      tone(BUZZER_PIN, NOTE_B2, 100, 0);
      reset_timer++;//подсчёт сколько раз не получается подключится
      Serial.println("connecting...");//вывод в Serial "подключение"
      //если не 60 раз не получается подключится
      if (reset_timer == 60) {
        tone(BUZZER_PIN, NOTE_C4, 2000, 5);//звуковой сигнал
        //установка режима "настройка"
        EEPROM.write(0, 0);
        EEPROM.commit();
        //перезагрузка
        ESP.restart();
      }
      delay(800);
    }
    //звуковой индекатор о удачном подключении
    tone(BUZZER_PIN, NOTE_C4, 500, 5);
    delay(100);
    noTone(BUZZER_PIN, 5);
    delay(200);
    tone(BUZZER_PIN, NOTE_F4, 500, 5);
    delay(100);
    noTone(BUZZER_PIN, 5);
    delay(200);
    tone(BUZZER_PIN, NOTE_B4, 500, 5);
    delay(100);
    noTone(BUZZER_PIN, 5);
    delay(200);
    //вывод IP адреса
    Serial.println(WiFi.localIP());
    //инициализация портов/
    pinMode(FLOOD_PIN, INPUT);
    pinMode(FIRE_PIN, INPUT);
    //колибровка датчика газов
    mq2.calibrate();
    //начинаем работу с датчиком температуры
    sensor.begin();
    //устанавливаем разрешение датчика от 9 до 12 бит
    sensor.setResolution(12);
    //устанывливаем подчёт не удач на 0
    reset_timer = 0;
  }
  else {
    //создание точки доступа
    WiFi.mode(WIFI_AP);
    WiFi.softAP("Module_3", "");
    //вывод IP
    Serial.print("AP IP address: ");
    Serial.println(WiFi.softAPIP());
    //установка настроек сервера
    mod.setupServer();
  }
}


void loop() {
  //если режим 1 - "работа"; если нет - "настройка"
  if (eeprom_status == 1) {
    //если потерена связь с WiFi - переподлключение
    if (WiFi.status() != WL_CONNECTED) {
      WiFi.disconnect();
      WiFi.begin(mod.getE(1, 2).c_str(), mod.getE(19, 20).c_str());
      while (WiFi.status() != WL_CONNECTED) {
        //оповещение о переподключении
        Serial.println("reconnecting...");
        //подсчёт сколько раз не получилось подключается
        reset_timer++;
        //если не получилось подключится 60 раз
        if (reset_timer == 60) {
          //режим "настройка" и перезагрузка
          EEPROM.write(0, 0);
          EEPROM.commit();
          ESP.restart();
        }
        delay(1000);
      }
    }
    //объявление WiFi  клиента
    WiFiClient client;

    //цикл пока не получится подлючится к главному модулю
    while (!client.connect(mod.getE(29, 30).c_str(), 80)) {
      tone(BUZZER_PIN, NOTE_B2, 100, 0);//звуковой индекатор
      //вывод IP главного модуля
      Serial.print("can't connect to ");
      Serial.println(mod.getE(29, 30));

      reset_timer ++;//подсчёт сколько раз не получается подключится
      //если не 60 раз не получается подключится
      if (reset_timer == 60) {
        //установка режима 0
        EEPROM.write(0, 0);
        EEPROM.commit();
        //перезагрузка
        ESP.restart();
      }
      delay(1000);
    }

    //если не 60 раз не получается что-то сделать
    if (reset_timer == 60) {
      //установка режима 0
      EEPROM.write(0, 0);
      EEPROM.commit();
      //перезагрузка
      ESP.restart();
    }
    //проверка датчиков каждые 3 сек
    if (millis() > intervalCheck + 3000) {
      Serial.println("check");
      //если датчик открытого огня показывает 0
      if (digitalRead(FIRE_PIN) == 0) {
        //индикатор огня и сделать отпрвку данных
        fire = 1;
        send_status = true;
      }
      else {
        fire = 0;
      }
      //если датчик влажности почвы больше 600
      if (analogRead(FLOOD_PIN) > 600) {
        //индикатор затопления и сделать отпрвку данных
        flood = 1;
        send_status = true;
      }
      else {
        flood = 0;
      }
      //если один из показателей датчика газов больше нормы
      if (mq2.readLPG() > 300 || mq2.readMethane() > 400 || mq2.readSmoke() > 220 || mq2.readHydrogen() > 300) {
        //индикатор утечки газа и сделать отпрвку данных
        gas_status = true;
        send_status = true;
      }
      else {
        gas_status = false;
      }
      //запоминание показанй датчика гозов(дым)
      gas = mq2.readSmoke();

      intervalCheck = millis();
    }
    //отправка данных каждые 7 сек или если индикатор отправки
    if (millis() > intervalSend + 7000 || send_status) {
      //запоминаие ответа от главного модуля
      String request = mod.wifiSend((String)"http://" + mod.getE(29, 30), (String)"/?second_temp=" + temp + "&second_flood=" + flood + "&second_fire=" + fire + "&second_gas=" + gas + "&second_gas_status=" + gas_status);
      //если нет ошибки - запомнить ответ в виде JSON
      if (request != "error") {
        deserializeJson(answer, request);
        serializeJson(answer, Serial);
        reset_timer = 0;
      }
      //ели есть ошибка
      else {
        reset_timer ++;
      }
      //если неоходимо отправить IP
      if (answer["get_ip_status"] == 1) {
        String str = "/?module=";
        str += "4";
        str += "&ip1=";
        str += String(WiFi.localIP()[0]);
        str += "&ip2=";
        str += String(WiFi.localIP()[1]);
        str += "&ip3=";
        str += String(WiFi.localIP()[2]);
        str += "&ip4=";
        str += String(WiFi.localIP()[3]);
        mod.wifiSend((String)"http://" + mod.getE(29, 30), str);
      }
      intervalSend = millis();
      send_status = false;
    }

  }
  else {
    //работа dns сервера
    dnsServer.processNextRequest();
  }
}
