//подключение библиотек
#include <ESPAsyncTCP.h>
#include "ESPAsyncWebServer.h"
#include <ArduinoJson.h>
#include <ESP8266WiFi.h>
#include <DNSServer.h>
#include <OtherModule.h>
//пины куда подключены датчики
#define row1 D5
#define row2 D6
#define row3 D7
//инициализируем классы
OtherModule mod;

//объявляем переменные

int8_t rezim = 0, eeprom_status, reset_timer = 0;

bool clean_pedal = false, send_status = false;

int8_t long_press = 0, on_press = 0, not_press = 0;
int long_press_timer = 0, on_press_timer = 0;
int intervalRead = 0;
DynamicJsonDocument answer(512);



//=====

void setup() {
  Serial.begin(9600);//инициализируем работу Serial-порта
  EEPROM.begin(128);//инициализируем работу EEPROM
  eeprom_status = EEPROM.read(0);//берём режим из памяти
  Serial.println(eeprom_status);//выводим его
  //если режим 1 - "работа"; если нет - "настройка"
  if (eeprom_status == 1) {
    //подключение к WiFI
    WiFi.mode(WIFI_STA);
    WiFi.begin(mod.getE(1, 2).c_str(), mod.getE(19, 20).c_str());
    while (WiFi.status() != WL_CONNECTED) {
      reset_timer++;//подсчет сколько раз не получилось подключится
      Serial.println("connecting...");
      //если не получилось подключится 60 раз
      if (reset_timer == 60) {
        //режим "настройка" и перезагрузка
        EEPROM.write(0, 0);
        EEPROM.commit();
        ESP.restart();
      }
      delay(1000);
    }
    Serial.println(WiFi.localIP());//вывод IP
    //инициализация портов
    pinMode(row1, INPUT);
    pinMode(row2, INPUT);
    pinMode(row3, INPUT);
    //устанывливаем подчёт не удач на 0
    reset_timer = 0;
  }
  else {
    //создание точки оступа
    WiFi.mode(WIFI_AP);
    WiFi.softAP("Module_2", "");

    Serial.print("AP IP address: ");
    Serial.println(WiFi.softAPIP());
    //настройка сервера
    mod.setupServer();
  }
}


void loop() {
  //если режим 1 - "работа"; если нет - "настройка"
  if (eeprom_status == 1) {
    WiFiClient client;
    //если потерена связь с WiFi - переподлключение
    if (WiFi.status() != WL_CONNECTED) {
      WiFi.disconnect();
      WiFi.begin(mod.getE(1, 2).c_str(), mod.getE(19, 20).c_str());
      while (WiFi.status() != WL_CONNECTED) {
        //оповещение о переподключении
        Serial.println("reconnecting...");
        //подсчёт сколько раз не получилось подключается
        reset_timer++;
        //если не получилось подключится 60 раз
        if (reset_timer == 60) {
          //режим 0 и перезагрузка
          EEPROM.write(0, 0);
          EEPROM.commit();
          ESP.restart();
        }
        delay(1000);
      }
    }
    //цикл если не получается подключится к клавному модулю
    while (!client.connect(mod.getE(29, 30).c_str(), 80)) {
      //оповещение о переподключении
      Serial.print("can't connect to ");
      Serial.println(mod.getE(29, 30));
      //подсчет сколько раз не получается подключится
      reset_timer ++;
      //если не получается подключится 60 раз
      if (reset_timer == 60) {
        //режим 0 и перезагрузка
        EEPROM.write(0, 0);
        EEPROM.commit();
        ESP.restart();
      }
      delay(1000);
    }
    //если не получается к чему-то подключится 60 раз
    if (reset_timer == 60) {
      //режим "настройка" и перезагрузка
      EEPROM.write(0, 0);
      EEPROM.commit();
      ESP.restart();
    }
    //отправка данных на главный модуль каждые 7 сек
    if (millis() > intervalRead + 7000 || send_status) {
      //запоминаем ответ
      String request = mod.wifiSend((String)"http://" + mod.getE(29, 30), (String)"/?long_press=" + EEPROM.read(51) + "&not_press=" + EEPROM.read(52) + "&on_press=" + EEPROM.read(53));
      //если нет ошибки - заповимаем ответ в виде JSON
      if (request != "error") {
        deserializeJson(answer, request);
        serializeJson(answer, Serial);
        reset_timer = 0;
      }
      else {
        reset_timer ++;
      }
      //если неоходимо отправить IP
      if (answer["get_ip_status"] == 1) {
        String str = "/?module=";
        str += "2";
        str += "&ip1=";
        str += String(WiFi.localIP()[0]);
        str += "&ip2=";
        str += String(WiFi.localIP()[1]);
        str += "&ip3=";
        str += String(WiFi.localIP()[2]);
        str += "&ip4=";
        str += String(WiFi.localIP()[3]);
        mod.wifiSend((String)"http://" + mod.getE(29, 30), str);
      }

      intervalRead = millis();
      send_status = false;
    }
    //если неоходимо очистить данные
    if (answer["cleen_pedal"] == 1) {
      EEPROM.write(51, 0);
      EEPROM.write(52, 0);
      EEPROM.write(53, 0);
      EEPROM.commit();
      answer["reset"] = 0;
    }
    //режим водителя - "работа"
    if (answer["rezim"] == 3) {
      //подсчет долгих нажатий длиной 5 сек
      if ( digitalRead(row1) == true && digitalRead(row2) == true && digitalRead(row3) == true && millis() > long_press_timer + 5000) {
        Serial.println("longpress");
        long_press ++;
        long_press_timer = millis();
      }
      else if ( digitalRead(row1) == false or digitalRead(row2) == false or digitalRead(row3) == false) {
        Serial.println("longpress_stop");
        long_press_timer = millis();
      }

      //подсчет не нажатий длиной 5 сек
      if ( digitalRead(row1) == false && digitalRead(row2) == false && digitalRead(row3) == false && millis() > on_press_timer + 5000) {
        Serial.println("unpress");
        on_press ++;
        on_press_timer = millis();
      }
      else if ( digitalRead(row1) == true or digitalRead(row2) == true or digitalRead(row3) == true) {
        Serial.println("unpress_stop");
        on_press_timer = millis();
      }

      //если 5 длянных нажатий - отправить эти данные на главный модуль
      if (long_press == 5) {
        EEPROM.write(51, EEPROM.read(51) + 1);
        EEPROM.commit();
        Serial.println("longpress_send");
        send_status = true;
        long_press = 0;
      }
      //если 5 не нажатий - отправить эти данные на главный модуль
      if (on_press == 5) {
        EEPROM.write(52, EEPROM.read(52) + 1);
        EEPROM.commit();
        Serial.println("unpress_send");
        send_status = true;
        on_press = 0;
      }
    }
     //режим водителя - "отдых" или "выбор режима водителя"
    else if (answer["rezim"] == 5 || answer["rezim"] == 6 || answer["rezim"] == 4 || answer["rezim"] == 1 || answer["rezim"] == 0 || answer["rezim"] == 2) {
      //подсчёт нажатий длиной 3 сек
      if ( digitalRead(row1) == true && millis() > on_press_timer + 3000 or digitalRead(row2) == true && millis() > on_press_timer + 3000 or digitalRead(row3) == true && millis() > on_press_timer + 3000) {
        Serial.println("press");
        on_press ++;
        on_press_timer = millis();
      }
      else if ( digitalRead(row1) == false and digitalRead(row2) == false and digitalRead(row3) == false) {
        Serial.println("press_stop");
        on_press_timer = millis();
      }

      //если 5 нажатий - отправить эти данные на главный модуль
      if (on_press == 5) {
        EEPROM.write(53, EEPROM.read(53) + 1);
        EEPROM.commit();
        Serial.println("press_send");
        send_status = true;
        on_press = 0;
      }
    }
  }
  else {
    //робота dns сервера
    dnsServer.processNextRequest();
  }
}
