//подключение библиотек
#include <AsyncTCP.h>
#include <Tone32.h>
#include "ESPAsyncWebServer.h"
#include <ArduinoJson.h>
#include <WiFi.h>
#include <DNSServer.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <OtherModule.h>
//пины куда подключены датчики и исполнительные элементы
#define TEMP_PIN 27
#define WATER_PIN 35
#define BUZZER_PIN 26
#define TRIG_FOOD_PIN 19
#define ECHO_FOOD_PIN 18
#define TRIG_DETECT_PIN 21
#define ECHO_DETECT_PIN 22

#define RELAY_COOL_PIN 4
#define RELAY_HOT_PIN 2
#define RELAY_FOOD_PIN 5
#define RELAY_WATER_PIN 15

//инициализируем классы
OtherModule mod;
OneWire oneWire(TEMP_PIN);
DallasTemperature sensor(&oneWire);

//объявление переменных

int8_t eeprom_status, reset_timer = 0;

int8_t temp, food, water;
int8_t no_food_count = 0, no_water_count = 0;

uint8_t dist_no_food, dist_no_detect;

bool send_status = false;
bool hot_status = false;

int intervalSend = 0;
int intervalCheck = 0;
DynamicJsonDocument answer(512);



//=====

void setup() {
  Serial.begin(9600);//инициализируем работу Serial-порта
  EEPROM.begin(128);//инициализируем работу EEPROM
  eeprom_status = EEPROM.read(0);//берём режим из памяти
  Serial.println(eeprom_status);//выводим его
  //если режим 1 - "работа"; если нет - "настройка"
  if (eeprom_status == 1) {
    //инициализируем порты (Ультразвуковые дальномеры)
    pinMode(TRIG_FOOD_PIN, OUTPUT);
    pinMode(ECHO_FOOD_PIN, INPUT);
    pinMode(TRIG_DETECT_PIN, OUTPUT);
    pinMode(ECHO_DETECT_PIN, INPUT);
    //подключение к WiFi
    WiFi.mode(WIFI_STA);
    WiFi.begin(mod.getE(1, 2).c_str(), mod.getE(19, 20).c_str());
    while (WiFi.status() != WL_CONNECTED) {
      //запоминание растояние до: нет еды, нет животног перед мисками
      dist_no_food = sonic(TRIG_FOOD_PIN, ECHO_FOOD_PIN);
      dist_no_detect = sonic(TRIG_DETECT_PIN, ECHO_DETECT_PIN);
      //оповещение о подключении
      tone(BUZZER_PIN, NOTE_B2, 100, 0);
      delay(200);
      tone(BUZZER_PIN, NOTE_B2, 100, 0);
      Serial.println("connecting...");
      //подсчет сколько раз не получилось подключится
      reset_timer++;
      //если не получилось подключится 60 раз
      if (reset_timer == 60) {
        //режим "настройка" и перезагрузка
        tone(BUZZER_PIN, NOTE_C4, 2000, 5);
        EEPROM.write(0, 0);
        EEPROM.commit();
        ESP.restart();
      }
      delay(800);
    }
    //оповещение о удачном подключении
    tone(BUZZER_PIN, NOTE_C4, 500, 5);
    delay(100);
    noTone(BUZZER_PIN, 5);
    delay(200);
    tone(BUZZER_PIN, NOTE_F4, 500, 5);
    delay(100);
    noTone(BUZZER_PIN, 5);
    delay(200);
    tone(BUZZER_PIN, NOTE_B4, 500, 5);
    delay(100);
    noTone(BUZZER_PIN, 5);
    delay(200);
    Serial.println(WiFi.localIP());
    //инициализируем порты
    pinMode(RELAY_COOL_PIN, OUTPUT);
    pinMode(RELAY_HOT_PIN, OUTPUT);
    pinMode(RELAY_FOOD_PIN, OUTPUT);
    pinMode(RELAY_WATER_PIN, OUTPUT);
    digitalWrite(RELAY_WATER_PIN, HIGH);
    digitalWrite(RELAY_FOOD_PIN, HIGH);
    digitalWrite(RELAY_HOT_PIN, HIGH);
    digitalWrite(RELAY_COOL_PIN, HIGH);
    pinMode(WATER_PIN, INPUT);
    //начинаем работу с датчиком температуры
    sensor.begin();
    //устанавливаем разрешение датчика от 9 до 12 бит
    sensor.setResolution(12);
    //устанывливаем подчёт не удач на 0
    reset_timer = 0;
  }
  else {
    //создание точки доступа
    WiFi.mode(WIFI_AP);
    WiFi.softAP("Module_3", "");
    //вывод IP
    Serial.print("AP IP address: ");
    Serial.println(WiFi.softAPIP());
    //установка настроек сервера
    mod.setupServer();
    //установка следующего режыма на 0(ничего)
    EEPROM.write(128, 0);
    EEPROM.commit();
  }
}


void loop() {
  //если режим 1 - "работа"; если нет - "настройка"
  if (eeprom_status == 1) {
    //если потерена связь с WiFi - переподлключение
    if (WiFi.status() != WL_CONNECTED) {
      WiFi.disconnect();
      WiFi.begin(mod.getE(1, 2).c_str(), mod.getE(19, 20).c_str());
      while (WiFi.status() != WL_CONNECTED) {
        //оповещение о переподключении
        Serial.println("reconnecting...");
        //подсчёт сколько раз не получилось подключается
        reset_timer++;
        //если не получилось подключится 60 раз
        if (reset_timer == 60) {
          //режим "настройка" и перезагрузка
          EEPROM.write(0, 0);
          EEPROM.commit();
          ESP.restart();
        }
        delay(1000);
      }
    }
    //объявление WiFi  клиента
    WiFiClient client;
    //цикл если не получается подключится к клавному модулю
    while (!client.connect(mod.getE(29, 30).c_str(), 80)) {
      //отключение всех реле (HIGH-отключение, LOW-включении)
      digitalWrite(RELAY_WATER_PIN, HIGH);
      digitalWrite(RELAY_FOOD_PIN, HIGH);
      digitalWrite(RELAY_HOT_PIN, HIGH);
      digitalWrite(RELAY_COOL_PIN, HIGH);
      //оповещение о переподключении
      Serial.print("can't connect to ");
      Serial.println(mod.getE(29, 30));
      
      //подсчет сколько раз не получается подключится
      reset_timer ++;
      //если не получается подключится 60 раз
      if (reset_timer == 60) {
        //режим 0 и перезагрузка
        EEPROM.write(0, 0);
        EEPROM.commit();
        ESP.restart();
      }
      delay(1000);
    }
    //если не получается к чему-то подключится 60 раз
    if (reset_timer == 60) {
      //режим "настройка" и перезагрузка
      EEPROM.write(0, 0);
      EEPROM.commit();
      ESP.restart();
    }
    //проверка датчиков каждые 3 сек
    if (millis() > intervalCheck + 3000) {
      Serial.println("check");
      //проверка есть ли вода
      if (analogRead(WATER_PIN) < 500 && sonic(TRIG_DETECT_PIN, ECHO_DETECT_PIN) >= dist_no_detect - 1) {
        Serial.println("noWATER");
        digitalWrite(RELAY_WATER_PIN, LOW);
        no_water_count ++;
      }
      else {
        digitalWrite(RELAY_WATER_PIN, HIGH);
        no_water_count = 0;
        water = 1;
      }
      //если не получается налить воды 15 раз - отправить сигнал, что нет воды
      if (no_water_count == 15) {
        Serial.println("no water 15");
        water = 0;
        send_status = true;
      }
      //проверка есть ли еда
      if (sonic(TRIG_FOOD_PIN, ECHO_FOOD_PIN) >= dist_no_food - 1 && sonic(TRIG_DETECT_PIN, ECHO_DETECT_PIN) >= dist_no_detect - 1) {
        Serial.println("noFOOOD");
        digitalWrite(RELAY_FOOD_PIN, LOW);
        no_food_count ++;
      }
      else {
        digitalWrite(RELAY_FOOD_PIN, HIGH);
        no_food_count - 0;
        food = 1;
      }
      //если не получается насыпать еды 15 раз - отправить сигнал, что нет воды
      if (no_food_count == 15) {
        Serial.println("no food 15");
        food = 0;
        send_status = true;
      }
      //запоминание температуры
      sensor.requestTemperatures();
      temp = sensor.getTempCByIndex(0);
      //если температура больше нужной то включить переключатель пельтье и выключить куллер
      if (temp < answer["needed_temp"]) {
        digitalWrite(RELAY_COOL_PIN, HIGH);
        hot_status = !hot_status;
      }
      //если больше нужной - включить куллер и выключить пельтье
      else {
        digitalWrite(RELAY_HOT_PIN, HIGH);
        digitalWrite(RELAY_COOL_PIN, LOW);
        hot_status = false;
      }
      intervalCheck = millis();
    }
    //отправка данных каждые 7 сек или если индикатор отправки
    if (millis() > intervalSend + 7000 || send_status) {
      //выключить пельтье
      digitalWrite(RELAY_HOT_PIN, HIGH);
      //запоминание ответа от гланого модуля
      String request = mod.wifiSend((String)"http://" + mod.getE(29, 30), (String)"/?first_temp=" + temp + "&first_food=" + food + "&first_water=" + water);
      //если нету ошибки - запомнить ответ в виде JSON
      if (request != "error") {
        deserializeJson(answer, request);
        serializeJson(answer, Serial);
        reset_timer = 0;
      }
      else {
        reset_timer ++;
      }
      intervalSend = millis();
      send_status = false;
    }
    //переклбчатель пельтье
    if (hot_status) {
      digitalWrite(RELAY_HOT_PIN, LOW);
    }
    else {
      digitalWrite(RELAY_HOT_PIN, HIGH);
    }

  }
  else {
    //рыбота dns сервера
    dnsServer.processNextRequest();
  }
}
//функция для получения данных от ультразвукового дальномера
int sonic(int8_t trig, int8_t echo) {
  int16_t cm = 0;
  digitalWrite(trig, LOW);
  delayMicroseconds(2);
  digitalWrite(trig, HIGH);
  delayMicroseconds(10);
  digitalWrite(trig, LOW);
  cm = pulseIn(echo, HIGH) / 58;
  Serial.println(cm);
  return cm;
}
